
import java.io.*;
import java.net.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;
import javax.net.ssl.*;

class APISERVexception extends java.lang.Exception {
}
     
public class APISERVclient {
    final public static String EROGATORE = "01234567890";
    final public static String BASE_URL = "http://127.0.0.1:8084/APISERV/";
    
    public static float getConsumo(String contatore) throws APISERVexception {
        float consumo;
        
        try {
            // costruzione dello URL di interrogazione e impostazione header richiesta web-service
            URL server = new URL(BASE_URL + "/consumo/" + URLEncoder.encode(contatore, "UTF-8") + "?erogatore=" + URLEncoder.encode(EROGATORE, "UTF-8"));
            HttpURLConnection service = (HttpURLConnection) server.openConnection();
            service.setRequestProperty("Accept-Charset", "UTF-8");
            service.setRequestProperty("Accept", "text/xml");
            service.setDoInput(true);
            service.setRequestMethod("GET");
            // connessione al web-service
            service.connect();
            // verifica stato risposta
            int status = service.getResponseCode();
            if (status != 200)
              throw new APISERVexception();
            // costruzione albero DOM del body XML
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            Document document = builder.parse(service.getInputStream());
            // estrazione del valore dell'elemento "consumo"
            Element root = document.getDocumentElement();
            if (!root.getNodeName().equalsIgnoreCase("consumo")) 
              throw new APISERVexception();
            String tipo = root.getAttribute("tipo");
            if (tipo.equalsIgnoreCase(""))
              throw new APISERVexception();
            String tmp = root.getFirstChild().getNodeValue();
            try {
                consumo = Float.valueOf(tmp);
            }
            catch (NumberFormatException e) {
                throw new APISERVexception();
            }
        }
        catch (IOException e) {
            throw new APISERVexception();
        }
        catch (ParserConfigurationException e) {
             throw new APISERVexception();
        }
        catch (SAXException e) {
             throw new APISERVexception();
        }
        return consumo;
    }
    
    public static void setAutolettura(String tipo, String contatore, float autolettura) throws APISERVexception {
        String tmp = Float.toString(autolettura);
        String body = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
        
        // costruzione body in formato XML
        body += "<autolettura tipo=\"" + tipo.substring(0, 1) + "\">" + autolettura + "</autolettura>\r\n";
        int n = body.length();
        try {
            // costruzione dello URL di interrogazione e impostazione header richiesta web-service
            URL server = new URL(BASE_URL + "/autolettura/" + URLEncoder.encode(contatore, "UTF-8") + "?erogatore=" + URLEncoder.encode(EROGATORE, "UTF-8"));
            HttpURLConnection service = (HttpURLConnection) server.openConnection();
            service.setRequestProperty("Content-type", "application/xml");
            service.setRequestProperty("Content-length", Integer.toString(n));
            service.setDoOutput(true);
            service.setRequestMethod("POST");
            // invio dati al web-service
            BufferedWriter output = new BufferedWriter(new OutputStreamWriter(service.getOutputStream(), "UTF-8"));
            output.write(body);
            output.flush();
            output.close();
            // connessione al web-service
            service.connect();
            // verifica stato risposta
            int status = service.getResponseCode();
            if (status != 201)
              throw new APISERVexception();
        }
        catch (IOException e) {
            throw new APISERVexception();
        }
    }
    
    public static void main(String[] args) {
        try {
            float consumo = APISERVclient.getConsumo("0123456789ABCDEF");
            System.out.println("Ultima lettura contatore \"0123456789ABCDEF\": " + consumo);
        }
        catch (APISERVexception e) {
            System.err.println("Errore interrogazione web-service.");
        }
        try {
            APISERVclient.setAutolettura("A", "0123456789ABCDEF", 2001);
            System.out.println("Autolettura effettuata.");
        }
        catch (APISERVexception e) {
            System.err.println("Errore interrogazione web-service.");
        }
    }
    
}
