
package it.acquagas.rete.APIAPP;

import java.io.*;
import java.net.*;
import java.sql.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;
import javax.net.ssl.*;

public class APIAPP extends HttpServlet {
    final static private String driver = "com.mysql.jdbc.Driver";
    final static private String dbms_url = "jdbc:mysql://127.0.0.1:3306/";
    final static private String database = "acqua_gas";
    final static private String user = "root";
    final static private String password = "";
    private static Connection connection;
    private static boolean connected;

    // attivazione servlet (connessione a DBMS)
    public void init() {
        String url = dbms_url+database;
        try {
            Class.forName(driver);
            connection = DriverManager.getConnection(url, user, password);
            connected = true;
        }
        catch (SQLException e) {
            connected = false;
        }
        catch (ClassNotFoundException e) {
            connected = false;
        }
    }
    
    // disattivazione servlet (disconnessione da DBMS)
    public void destroy() {
        try {
            connection.close();
        }
        catch (SQLException e) {
        }
    }

    // verifica API-key
    private void testKey(String APIkey, HttpServletResponse response) throws IOException {
        if (APIkey == null) {
            response.sendError(400, "Missing API key!");
            return;
        }
        try {
            Statement statement = connection.createStatement();
            ResultSet resultset = statement.executeQuery( "SELECT * FROM Dispositivo WHERE API_key = '" + APIkey + "';");
            if (resultset.next()) {
            }
            else {
                response.sendError(400, "Wrong API key!");
                resultset.close();
                statement.close();
                return;
            }
            resultset.close();
            statement.close();
        }
        catch (SQLException e) {
            response.sendError(500, "DBMS server error!");
        }
    }
    
    
    
    
    private void getMap(String contatore, HttpServletResponse response) throws IOException {
        double latitudine = 0.0, longitudine= 0.0;
        String prefix = "https://maps.googleapis.com/maps/api/staticmap?center=";
        String suffix = "&zoom=20&size=512x512&maptype=hybrid&markers=";
           
        try {
            // ricerca dati contatore nel database
            Statement statement = connection.createStatement();
            String query = "SELECT Latitudine, Longitudine FROM Contatore WHERE Contatore.ID_contatore = '" + contatore + "';";
            ResultSet resultset = statement.executeQuery(query);
            if (resultset.next()) {
                latitudine = resultset.getDouble(1);
                longitudine = resultset.getDouble(2);
            }
            else {
                response.sendError(404, "Not found!");
                resultset.close();
                statement.close();
                return;
            }
            resultset.close();
            statement.close();
        }
        catch (SQLException e) {
            response.sendError(500, "DBMS server error!");
        }
        String url = prefix + latitudine + "," + longitudine + suffix + latitudine + "," + longitudine;
        URL server = new URL(url);
        HttpsURLConnection service = (HttpsURLConnection) server.openConnection();
        service.setRequestProperty("Accept", "image/png");
        service.setDoInput(true);
        service.setRequestMethod("GET");
        // connessione al web-service
        service.connect();
        // verifica stato risposta
        int status = service.getResponseCode();
        if (status != 200) {
            response.sendError(500, "Google web-service error!");
        }
        // scrittura del body della risposta
        response.setContentType("image/png");
        OutputStream out = response.getOutputStream();
        InputStream in = service.getInputStream();
        byte[] buffer = new byte[65536];
        int n;
        while ((n = in.read(buffer)) >= 0)
            out.write(buffer, 0, n);
        in.close();
        out.flush();
        out.close();
        response.setStatus(200); // OK
    }
    
    
    // richiesta dati cliente contatore
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
                   throws ServletException, IOException {
        String url, contatore, APIkey;
        String[] url_section;
        String CF, cognome, nome, indirizzo, numero, CAP, località, comune, provincia;
        // verifica stato connessione a DBMS
        if (!connected) {
            response.sendError(500, "DBMS server error!");
            return;
        }
        
        // verifica API-key
        APIkey = request.getParameter("key");
        testKey(APIkey, response);
        
        // verifica identificatore contatore
        url = request.getRequestURL().toString();
        url_section = url.split("/");
        if (url_section.length < 2) {
            response.sendError(400, "Malformed URL!");
            return;
        }
        contatore = url_section[url_section.length-1];
        if (contatore == null) {
            response.sendError(400, "Request syntax error!");
            return;
        }
        if (!url_section[url_section.length-2].equalsIgnoreCase("cliente")) {
            if (url_section[url_section.length-2].equalsIgnoreCase("mappa")) {
                getMap(contatore, response);
                return;
            }
            response.sendError(405, "Wrong request!");
            return;
        }
        if (contatore.isEmpty()) {
            response.sendError(400, "Request syntax error!");
            return;
        }
        try {
            // ricerca dati cliente nel database
            Statement statement = connection.createStatement();
            String query = "SELECT * FROM Contatore, Cliente WHERE Contatore.Cliente = Cliente.Codice_fiscale AND Contatore.ID_contatore = '" + contatore + "';";
            ResultSet resultset = statement.executeQuery(query);
            if (resultset.next()) {
                CF = resultset.getString("Cliente.Codice_fiscale");
                cognome = resultset.getString("Cliente.Cognome");
                nome = resultset.getString("Cliente.Nome");
                indirizzo = resultset.getString("Cliente.Indirizzo");
                numero = resultset.getString("Cliente.Numero");
                località = resultset.getString("Cliente.Località");
                CAP = resultset.getString("Cliente.CAP");
                comune = resultset.getString("Cliente.Comune");
                provincia = resultset.getString("Cliente.Provincia");
            }
            else {
                response.sendError(404, "Not found!");
                resultset.close();
                statement.close();
                return;
            }
            resultset.close();
            statement.close();
            // scrittura del body della risposta
            response.setContentType("text/xml;charset=UTF-8");
            PrintWriter out = response.getWriter();
            try {
                out.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n");
                out.println("<cliente ID_contatore=\"" + contatore + "\">\r\n");
                out.print("<CF>");
                out.print(CF);
                out.println("</CF>\r\n");
                out.print("<cognome>");
                out.print(cognome);
                out.println("</cognome>\r\n");
                out.print("<nome>");
                out.print(nome);
                out.println("</nome>\r\n");
                out.print("<indirizzo>");
                out.print(indirizzo);
                out.println("</indirizzo>\r\n");
                out.print("<numero>");
                out.print(numero);
                out.println("</numero>\r\n");
                out.print("<località>");
                out.print(località);
                out.println("</località>\r\n");
                out.print("<CAP>");
                out.print(CAP);
                out.println("</CAP>\r\n");
                out.print("<comune>");
                out.print(comune);
                out.println("</comune>\r\n");
                out.print("<provincia>");
                out.print(provincia);
                out.println("</provincia>");
                out.println("</cliente>");
            }
            finally {
                out.close();
            }
            response.setStatus(200); // OK
        }
        catch (SQLException e) {
            response.sendError(500, "DBMS server error!");
        }
    }

    // inoltro dati contatore
    protected void doPost (HttpServletRequest request, HttpServletResponse response)
              throws ServletException, IOException {
        String url, contatore, APIkey, tipo;
        String[] url_section;
        float lettura = 0;
        
        // verifica stato connessione a DBMS
        if (!connected) {
            response.sendError(500, "DBMS server error!");
            return;
        }
        
        // verifica API-key
        APIkey = request.getParameter("key");
        testKey(APIkey, response);
        
        // verifica identificatore contatore
        url = request.getRequestURL().toString();
        url_section = url.split("/");
        if (url_section.length < 2) {
            response.sendError(400, "Malformed URL!");
            return;
        }
        contatore = url_section[url_section.length-1];
        if (contatore == null) {
            response.sendError(400, "Request syntax error!");
            return;
        }
        if (!url_section[url_section.length-2].equalsIgnoreCase("lettura")) {
            response.sendError(405, "Wrong request!");
            return;
        }
        if (contatore.isEmpty()) {
            response.sendError(400, "Request syntax error!");
            return;
        }
        /*
        try {
            Statement statement = connection.createStatement();
            ResultSet resultset = statement.executeQuery( "SELECT * FROM Contatore WHERE ID_contatore = '" + contatore + "';");
            if (resultset.next()) {
            }
            else {
                response.sendError(400, "Wrong meter ID!");
                resultset.close();
                statement.close();
                return;
            }
            resultset.close();
            statement.close();
        }
        catch (SQLException e) {
            response.sendError(500, "DBMS server error!");
        }
        */
        try {
            // estrazione dell'attributo "tipo" e del valore dell'elemento "lettura" dal body XML
            BufferedReader content = request.getReader();
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            InputSource input = new InputSource();
            input.setCharacterStream(content);
            Document document = builder.parse(input);
            Element root = document.getDocumentElement();
            if (!root.getNodeName().equalsIgnoreCase("lettura")) {
                response.sendError(400, "Wrong request body!");
            }
            tipo = root.getAttribute("tipo");
            if (!tipo.equalsIgnoreCase("A") && !tipo.equalsIgnoreCase("G")) {
                response.sendError(400, "Wrong request body!");
            }
            String tmp = root.getFirstChild().getNodeValue();
            try {
                lettura = Float.valueOf(tmp);
            }
            catch (NumberFormatException e) {
                response.sendError(400, "Wrong request body!");
            }
            try {
                String command = "INSERT Lettura(Contatore, Data_ora, Consumo, Dispositivo) VALUES('" + contatore + "', NOW(), " + Float.toString(lettura) + ", '" + APIkey + "');";
                Statement statement = connection.createStatement();
                if (statement.executeUpdate(command) <= 0) {
                    statement.close();
                    //response.sendError(500, "DBMS server error!");
                    response.sendError(404, "Wrong meter ID?");
                }
                statement.close();
            }
            catch (SQLException e) {
                response.sendError(500, "DBMS server error!");
            }
            response.setStatus(201); // OK CREATED
        }
        catch (ParserConfigurationException e) {
            response.sendError(500, "XML parser error!");
        }
        catch (SAXException e) {
            response.sendError(500, "XML parser error!");
        }
    }
    
    // inoltro posizione contatore
    protected void doPut (HttpServletRequest request, HttpServletResponse response)
              throws ServletException, IOException {
        String url, contatore, APIkey;
        String[] url_section;
        double latitudine = 0.0;
        double longitudine = 0.0;
        
        // verifica stato connessione a DBMS
        if (!connected) {
            response.sendError(500, "DBMS server error!");
            return;
        }
        
        // verifica API-key
        APIkey = request.getParameter("key");
        testKey(APIkey, response);
        
        // verifica identificatore contatore
        url = request.getRequestURL().toString();
        url_section = url.split("/");
        if (url_section.length < 2) {
            response.sendError(400, "Malformed URL!");
            return;
        }
        contatore = url_section[url_section.length-1];
        if (contatore == null) {
            response.sendError(400, "Request syntax error!");
            return;
        }
        if (!url_section[url_section.length-2].equalsIgnoreCase("lettura")) {
            response.sendError(405, "Wrong request!");
            return;
        }
        if (contatore.isEmpty()) {
            response.sendError(400, "Request syntax error!");
            return;
        }
        /*
        try {
            Statement statement = connection.createStatement();
            ResultSet resultset = statement.executeQuery( "SELECT * FROM Contatore WHERE ID_contatore = '" + contatore + "';");
            if (resultset.next()) {
            }
            else {
                response.sendError(400, "Wrong meter ID!");
                resultset.close();
                statement.close();
                return;
            }
            resultset.close();
            statement.close();
        }
        catch (SQLException e) {
            response.sendError(500, "DBMS server error!");
        }
        */
        try {
            // estrazione dell'attributo "tipo" e del valore dell'elemento "lettura" dal body XML
            BufferedReader content = request.getReader();
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            InputSource input = new InputSource();
            input.setCharacterStream(content);
            Document document = builder.parse(input);
            Element root = document.getDocumentElement();
            if (!root.getNodeName().equalsIgnoreCase("posizione")) {
                response.sendError(400, "Wrong request body!");
            }
            NodeList list = root.getElementsByTagName("latitudine");
            String tmp = null;
            if (list != null && list.getLength() > 0)
                tmp = list.item(0).getFirstChild().getNodeValue();
            try {
                latitudine = Double.valueOf(tmp);
            }
            catch (NumberFormatException e) {
                response.sendError(400, "Wrong request body!");
            }
            list = root.getElementsByTagName("longitudine");
            tmp = null;
            if (list != null && list.getLength() > 0)
                tmp = list.item(0).getFirstChild().getNodeValue();
            try {
                longitudine = Double.valueOf(tmp);
            }
            catch (NumberFormatException e) {
                response.sendError(400, "Wrong request body!");
            }
            try {
                String command = "UPDATE Contatore SET Latitudine = " + Double.toString(latitudine) + ", Longitudine = " + Double.toString(longitudine) + " WHERE ID_contatore = '" + contatore + "';";
                Statement statement = connection.createStatement();
                if (statement.executeUpdate(command) <= 0) {
                    statement.close();
                    //response.sendError(500, "DBMS server error!");
                    response.sendError(404, "Wrong meter ID?");
                }
                statement.close();
            }
            catch (SQLException e) {
                response.sendError(500, "DBMS server error!");
            }
            response.setStatus(204); // OK NO-CONTENT
        }
        catch (ParserConfigurationException e) {
            response.sendError(500, "XML parser error!");
        }
        catch (SAXException e) {
            response.sendError(500, "XML parser error!");
        }
    }
    
    public String getServletInfo() {
        return "APIAPP servlet";
    }

}
